#include "MDR32F9Qx_rst_clk.h"
#include "MDR32F9Qx_adc.h"
#include "uart.h"
#include "itmr.h"
#include "cb.h"
#include "MDR32F9Qx_port.h"
#define mADC_RES_DATA (0x00000FFFul)

#define CHAN_NUM (2)

#define BUF_SIZE 32
static char SampleBuf[BUF_SIZE];
static TCB SampleCB;
static volatile char StopConvert;
#define CONVERT_PERIOD (1000)
static unsigned int ConvertPeriod;

static unsigned int ADCState;
enum{
	AS_IDLE,
	AS_START,
	AS_WAIT_RES,
	AS_CNT
};

//--------------------------------------------------------
void adc_Init(void)
{		
	MDR_RST_CLK->PER2_CLOCK |= (1ul << 8);	// 7 for DAC
	MDR_RST_CLK->ADC_MCO_CLOCK = 0x00002FF0;	//0x00002F20
	//ADC->ADC1_CFG |= 0xFEC2F025;
	MDR_ADC->ADC1_CFG |= 0xFEC2500D | (0 << 4);	// 0 channel
	MDR_ADC->ADC1_STATUS = 0x00000010;		
	cb_Init(&SampleCB, SampleBuf, BUF_SIZE);	
	ConvertPeriod = itmr_GetTick();
	//NVIC_EnableIRQ(ADC_IRQn);			

	
	/*RST_CLK_PCLKcmd(RST_CLK_PCLK_ADC, ENABLE);
	ADC_DeInit();
	ADC_StructInit(&ADC_InitStructure);
	ADC_InitStructure.ADC_SynchronousMode = ADC_SyncMode_Independent;
	ADC_InitStructure.ADC_TempSensor = ADC_TEMP_SENSOR_Enable;
	ADC_Init(&ADC_InitStructure);
	ADCx_StructInit(&ADCx_InitStruture);
	ADCx_InitStructure.ADC_SamplingMode = ADC_SAMPLING_MODE_CICLIC_CONV;
	ADCx_InitStructure.ADC_CHANNELNUMBER = ADC_CH_ADC1;
	ADC1_Init(&ADCx_InitStructure);*/
	
}

//--------------------------------------------------------
void adc_IRQ(void)
{				
	unsigned int tmp = MDR_ADC->ADC1_RESULT;	
	tmp &= mADC_RES_DATA;		
	cb_AddByte(&SampleCB, (char)(tmp & 0xff));	
	cb_AddByte(&SampleCB, (char)((tmp >> 8) & 0xff));		
	StopConvert = 1;
	
}

//--------------------------------------------------------
static char IsSample(void)
{
	unsigned int res;
///	NVIC_DisableIRQ(ADC_IRQn);
	res = cb_GetByteCnt(&SampleCB);	
//	NVIC_EnableIRQ(ADC_IRQn);	
	return (res > 3) ? 1 : 0;
}

//--------------------------------------------------------
static unsigned int GetSample(void)
{	
	char byte[3];
//	NVIC_DisableIRQ(ADC_IRQn);
	cb_GetByte(&SampleCB, &byte[0]);
	cb_GetByte(&SampleCB, &byte[1]);
	//cb_GetByte(&SampleCB, &byte[2]);	
//	NVIC_EnableIRQ(ADC_IRQn);	
	return byte[0] | (byte[1] << 8);
}

//--------------------------------------------------------
char adc_GetSample(unsigned int* sample)
{
	if(IsSample()){
		*sample = GetSample();
		return 1;
	}
	else
		return 0;
}

//--------------------------------------------------------
static void adc_StartConvert(void)
{
	 MDR_ADC->ADC1_CFG |= (1 << 1);
}
	 
//--------------------------------------------------------
char adc_Work(unsigned int* psample)
{		
	#define CONV_CNT	100
	static unsigned int conv_cnt;
	static unsigned int sample, sum;
	switch(ADCState){
		case AS_IDLE:
			if(itmr_IsTime(ConvertPeriod, CONVERT_PERIOD)){
				ADCState = AS_START;
				ConvertPeriod = itmr_GetTick();
				conv_cnt = CONV_CNT;				
				sum = 0;
			}
			break;
		case AS_START:
			adc_StartConvert();
			StopConvert = 0;	
			conv_cnt--;
			ADCState = AS_WAIT_RES;		
			break;
		case AS_WAIT_RES:
			if(StopConvert){
				adc_GetSample(&sample);
				sum += sample;
				if(conv_cnt)
					ADCState = AS_START;
				else{
					ADCState = AS_IDLE;
					*psample = sum / CONV_CNT;
					return 1;
				}
			}
			break;
		default:
			break;
	}
	return 0;
}

unsigned int adc_GetMaxVal(void)
{
	return 0x00000FFF;
}
