#ifndef __PROT_HRD_H__
#define __PROT_HDR_H__

//----------------------------------------------------
// MAC - 
//  Ethernet
typedef struct {unsigned char addr[6];} TEthAddr;

//  Ethernet
typedef struct{
	TEthAddr	dstMAC;		//  
	TEthAddr	srcMAC;		//  
	unsigned short	type;	//  
} TEthernetHeader, *pEthernetHeader;
#define ETHERNET_HDR_SIZE	sizeof(TEthernetHeader)

//  
#define ETHTYPE_ARP		0x0608	  //   0x0806
#define ETHTYPE_IPv4	0x0008	  //   0x0800
#define ETHTYPE_IPv6	0xDD86	  //   0x08dd

//----------------------------------------------------
// IP - 
//  IPv4
typedef union {
	unsigned char	d8[4];			// 
	unsigned short	d16[2];
} TIP4Addr;

//  IP
typedef struct{
	unsigned char version;	//   IP   4,   IP   unsigned int
	unsigned char service;	//  
	unsigned short length;	//  
	unsigned short id;		// 
	unsigned short off;		// ,        3  
	unsigned char ttl;		//   
	unsigned char prot;		//   .  (TCP, UDP...)
	unsigned short cs;		//   
	TIP4Addr src_ip;		// IP  
	TIP4Addr dst_ip;		// IP  
} TIPHeader, *pTIPHeader;
#define IP_HEADER_SIZE	sizeof(TIPHeader)

//   over head
enum{
	POH_ICMP = 1,
	POH_IGMP = 2,
	POH_TCP = 6,
	POH_UDP = 17
};

//  (UDP  TCP),   
typedef struct{
	TIP4Addr src_addr;			//  
	TIP4Addr dst_addr;			//  
	unsigned short protocol;	// 
	unsigned short length;		//   UDP
} TPseudoHdr, *pTPseudoHdr;
#define PSEUDO_HDR_SIZE sizeof(TPseudoHdr)

//----------------------------------------------------
// ARP - 
//  ARP 
typedef struct{
	unsigned short hrdw_type;	//   
	unsigned short prot_type;	//  	
	unsigned char hrdw_addr_length;	//     
	unsigned char prot_addr_length; //     
	unsigned short op_code;		//    : 1 - , 2 - 
	TEthAddr src_hrdw_addr;	//   
	TIP4Addr src_prot_addr;	//   	
	TEthAddr dst_hrdw_addr;	//   
	TIP4Addr dst_prot_addr; //   
} TARPPack, *pTARPPack;
#define ARP_HEADER_SIZE	sizeof(TARPPack)

//   
enum{
	HRDW_TRAN_PROT_ETHERNET	= 0x0100
};

//  
enum{
	PROT_CODE_IPv4 = 0x0008 //0x0800				
};

//   
enum{
	HRDW_ADDR_LENGTH_ETHERNET = 6
};

//   
enum{
	LGC_ADDR_LENGTH_IPv4 = 4
};

//  
enum{
	ARP_OP_CODE_REQUEST = 0x0100,	// 
	ARP_OP_CODE_ANSWER = 0x0200			// 
};

//----------------------------------------------------
// ICMP - 
typedef struct{
	unsigned char type;	//  
	unsigned char code;	// 
	unsigned short cs;	//  
} TICMPHeader, *pTICMPHeader;
#define ICMP_HEADER_SIZE sizeof(TICMPHeader)

#define TYPE_ECHO_REPLY			0
	#define CODE_ECHO_REPLY		0

#define TYPE_ECHO_REQUEST		8
	#define CODE_ECHO_REQUEST	0

enum{
	CODE_ERROR,
	CODE_REQUEST,
	CODE_REPLY
};

//   ICMP 
#define ICMP_DATA_SIZE	36

//----------------------------------------------------
// UDP - 
//  UDP
typedef struct{		
	unsigned short src_port;	//  
	unsigned short dst_port;	//  
	unsigned short length;		//  
	unsigned short cs;			//    
} TUDPHeader, *pTUDPHeader;
//   UDP
#define	UDP_HEADER_SIZE	sizeof(TUDPHeader)

//----------------------------------------------------
//  
// ETH-IP
typedef struct{
	TEthernetHeader eth;
	TIPHeader ip;		
} TETHIPHeader, *pTETHIPHeader;
#define ETH_IP_HEADER_SIZE	sizeof(TETHIPHeader)

// IP - ICMP
typedef struct{
	TIPHeader ip;
	TICMPHeader icmp;
} TIPICMPHeader, *pTIPICMPHeader;

#define IP_ICMP_HEADER_SIZE 	sizeof(TIPICMPHeader)

// IP - UDP
typedef struct{
	TIPHeader ip;
	TUDPHeader udp;
} TIPUDPHeader, *pTIPUDPHeader;

#define IP_UDP_HEADER_SIZE 		sizeof(TIPUDPHeader)
//----------------------------------------------------
#define ETH_HDR(buf)		((pEthernetHeader)(buf))
#define ARP_HDR(buf)		((pTARPPack)(buf + ETHERNET_HDR_SIZE))
#define IP_HDR(buf)			((pTIPHeader)(buf + ETHERNET_HDR_SIZE))
#define ICMP_HDR(buf)		((pTICMPHeader)(buf + ETH_IP_HEADER_SIZE))
#define UDP_HDR(buf)		((pTUDPHeader)(buf + ETH_IP_HEADER_SIZE))
#define IPICMP_HDR(buf)		((pTIPICMPHeader)(buf + ETHERNET_HDR_SIZE))
#define IPUDP_HDR(buf)		((pTIPUDPHeader)(buf + ETHERNET_HDR_SIZE))

#endif
