#include <string.h>
#include "itmr.h"
#include "console.h"
#include "wnds.h"
#include "disp.h"
#include "eth_def.h"
#include "eth_drv.h"

#define SPEED_SEL_CNT 		(2)

static TEthSettings EthSettings;

static char* selSpeedText[SPEED_SEL_CNT] = { "10/", "100/" };
static TWESelector selSpeed = { selSpeedText, SPEED_SEL_CNT, 0 };

BEGIN_WE_TABLE(ElemTable)
	DEFINE_WE(WET_SELECTOR,	0, 0, 0, ": ", &selSpeed)
	DEFINE_WE(WET_BUTTON, 0, 1, 0, "MAC ", 0)  	
	DEFINE_WE(WET_BUTTON, 0, 2, 0, "IP ", 0)  
	DEFINE_WE(WET_BUTTON, 0, 3, 0, " ", 0)  
	DEFINE_WE(WET_BUTTON, 0, 4, 0, "", 0)  	
END_WE_TABLE()

static bool EventProc(unsigned int code, unsigned int data);

static TWnd Wnd = { WID_ETHERNET, ElemTable, EventProc, 0 };

static char GetSpeed(void)
{
	switch(EthSettings.speed){
		case PHY_MODE_10BaseT_FDn:
			return 0;
		case PHY_MODE_100BaseT_FDn:
			return 1;
		default:
			return 0;
	}
}

static void SetSpeed(char speed)
{
	switch(speed){
		case 0:
			EthSettings.speed = PHY_MODE_10BaseT_FDn;
			break;
		case 1:
			EthSettings.speed = PHY_MODE_100BaseT_FDn;
			break;
		default:
			EthSettings.speed = PHY_MODE_10BaseT_FDn;
			break;
	}
}

static bool EventProc(unsigned int code, unsigned int data)
{
  switch (code) {
    case WE_DRAW:
      if (data == 0) {				
				disp_ShowStr(0, 5, " ping");				
				disp_ShowStr(0, 6, "  -Reset");	
				selSpeed.cur_val = GetSpeed();				
      }
      break;  
		case WE_EDIT_END:
			if(data){	
				switch(Wnd.cur_elem){
					case 0:
						SetSpeed(selSpeed.cur_val);
						ethernet_SetSettings(&EthSettings);
						break;
					default:
						break;
				}
			}
			break;
		case WE_ELEM_ACT:
			switch(Wnd.cur_elem){
				case 1:
					wnd_EthMAC();
					break;				
				case 2:
					wnd_EthIP();
					break;
				case 3:
					wnd_EthMASK();
					break;
				case 4:
					wnd_EthGATEWAY();
					break;
				default:
					break;
			}
  }

  return false;
}

void wnd_Ethernet(void)
{ 
	ethernet_GetSettings(&EthSettings);	
	console_OpenWnd(&Wnd);
}
